from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define, field as _attrs_field

if TYPE_CHECKING:
    from ..models.incident_group_schema import IncidentGroupSchema
    from ..models.page_info import PageInfo


T = TypeVar("T", bound="PagedIncidentGroupSchema")


@_attrs_define
class PagedIncidentGroupSchema:
    """
    Attributes:
        items (list['IncidentGroupSchema']):
        page_info (PageInfo):
    """

    items: list["IncidentGroupSchema"]
    page_info: "PageInfo"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        items = []
        for items_item_data in self.items:
            items_item = items_item_data.to_dict()
            items.append(items_item)

        page_info = self.page_info.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "items": items,
                "page_info": page_info,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.incident_group_schema import IncidentGroupSchema
        from ..models.page_info import PageInfo

        d = src_dict.copy()
        items = []
        _items = d.pop("items")
        for items_item_data in _items:
            items_item = IncidentGroupSchema.from_dict(items_item_data)

            items.append(items_item)

        page_info = PageInfo.from_dict(d.pop("page_info"))

        paged_incident_group_schema = cls(
            items=items,
            page_info=page_info,
        )

        paged_incident_group_schema.additional_properties = d
        return paged_incident_group_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define, field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.schedule_shift_user_schema import ScheduleShiftUserSchema


T = TypeVar("T", bound="ScheduleShiftSchema")


@_attrs_define
class ScheduleShiftSchema:
    """
    Attributes:
        start (datetime.datetime):
        end (datetime.datetime):
        users (list['ScheduleShiftUserSchema']):
    """

    start: datetime.datetime
    end: datetime.datetime
    users: list["ScheduleShiftUserSchema"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        start = self.start.isoformat()

        end = self.end.isoformat()

        users = []
        for users_item_data in self.users:
            users_item = users_item_data.to_dict()
            users.append(users_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "start": start,
                "end": end,
                "users": users,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.schedule_shift_user_schema import ScheduleShiftUserSchema

        d = src_dict.copy()
        start = isoparse(d.pop("start"))

        end = isoparse(d.pop("end"))

        users = []
        _users = d.pop("users")
        for users_item_data in _users:
            users_item = ScheduleShiftUserSchema.from_dict(users_item_data)

            users.append(users_item)

        schedule_shift_schema = cls(
            start=start,
            end=end,
            users=users,
        )

        schedule_shift_schema.additional_properties = d
        return schedule_shift_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field

from ..models.incident_urgency import IncidentUrgency
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateHeartbeatInput")


@_attrs_define
class CreateHeartbeatInput:
    """
    Attributes:
        name (str):
        description (str):
        service (int):
        link (Union[None, Unset, str]):
        incident_urgency (Union[IncidentUrgency, None, Unset]):
        labels (Union[Unset, list[str]]):
    """

    name: str
    description: str
    service: int
    link: Union[None, Unset, str] = UNSET
    incident_urgency: Union[IncidentUrgency, None, Unset] = UNSET
    labels: Union[Unset, list[str]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description = self.description

        service = self.service

        link: Union[None, Unset, str]
        if isinstance(self.link, Unset):
            link = UNSET
        else:
            link = self.link

        incident_urgency: Union[None, Unset, str]
        if isinstance(self.incident_urgency, Unset):
            incident_urgency = UNSET
        elif isinstance(self.incident_urgency, IncidentUrgency):
            incident_urgency = self.incident_urgency.value
        else:
            incident_urgency = self.incident_urgency

        labels: Union[Unset, list[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "description": description,
                "service": service,
            }
        )
        if link is not UNSET:
            field_dict["link"] = link
        if incident_urgency is not UNSET:
            field_dict["incident_urgency"] = incident_urgency
        if labels is not UNSET:
            field_dict["labels"] = labels

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description")

        service = d.pop("service")

        def _parse_link(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        link = _parse_link(d.pop("link", UNSET))

        def _parse_incident_urgency(data: object) -> Union[IncidentUrgency, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                incident_urgency_type_0 = IncidentUrgency(data)

                return incident_urgency_type_0
            except:  # noqa: E722
                pass
            return cast(Union[IncidentUrgency, None, Unset], data)

        incident_urgency = _parse_incident_urgency(d.pop("incident_urgency", UNSET))

        labels = cast(list[str], d.pop("labels", UNSET))

        create_heartbeat_input = cls(
            name=name,
            description=description,
            service=service,
            link=link,
            incident_urgency=incident_urgency,
            labels=labels,
        )

        create_heartbeat_input.additional_properties = d
        return create_heartbeat_input

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define, field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.schedule_override_user_schema import ScheduleOverrideUserSchema


T = TypeVar("T", bound="ScheduleOverrideSchema")


@_attrs_define
class ScheduleOverrideSchema:
    """
    Attributes:
        user (ScheduleOverrideUserSchema):
        start (datetime.datetime):
        end (datetime.datetime):
        id (Union[None, Unset, int]):
    """

    user: "ScheduleOverrideUserSchema"
    start: datetime.datetime
    end: datetime.datetime
    id: Union[None, Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        user = self.user.to_dict()

        start = self.start.isoformat()

        end = self.end.isoformat()

        id: Union[None, Unset, int]
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user": user,
                "start": start,
                "end": end,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.schedule_override_user_schema import ScheduleOverrideUserSchema

        d = src_dict.copy()
        user = ScheduleOverrideUserSchema.from_dict(d.pop("user"))

        start = isoparse(d.pop("start"))

        end = isoparse(d.pop("end"))

        def _parse_id(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        id = _parse_id(d.pop("id", UNSET))

        schedule_override_schema = cls(
            user=user,
            start=start,
            end=end,
            id=id,
        )

        schedule_override_schema.additional_properties = d
        return schedule_override_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.schedule_shift_schema import ScheduleShiftSchema
from ...types import UNSET, Response, Unset


def _get_kwargs(
    schedule_id: int,
    *,
    datetime_start: datetime.datetime,
    datetime_end: datetime.datetime,
    overflow: Union[Unset, bool] = True,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_datetime_start = datetime_start.isoformat()
    params["datetime_start"] = json_datetime_start

    json_datetime_end = datetime_end.isoformat()
    params["datetime_end"] = json_datetime_end

    params["overflow"] = overflow

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/v1/schedules/{schedule_id}/shifts/".format(
            schedule_id=schedule_id,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[list["ScheduleShiftSchema"]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = ScheduleShiftSchema.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[list["ScheduleShiftSchema"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    schedule_id: int,
    *,
    client: AuthenticatedClient,
    datetime_start: datetime.datetime,
    datetime_end: datetime.datetime,
    overflow: Union[Unset, bool] = True,
) -> Response[list["ScheduleShiftSchema"]]:
    """List Schedule Shifts

    Args:
        schedule_id (int):
        datetime_start (datetime.datetime):
        datetime_end (datetime.datetime):
        overflow (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['ScheduleShiftSchema']]
    """

    kwargs = _get_kwargs(
        schedule_id=schedule_id,
        datetime_start=datetime_start,
        datetime_end=datetime_end,
        overflow=overflow,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    schedule_id: int,
    *,
    client: AuthenticatedClient,
    datetime_start: datetime.datetime,
    datetime_end: datetime.datetime,
    overflow: Union[Unset, bool] = True,
) -> Optional[list["ScheduleShiftSchema"]]:
    """List Schedule Shifts

    Args:
        schedule_id (int):
        datetime_start (datetime.datetime):
        datetime_end (datetime.datetime):
        overflow (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['ScheduleShiftSchema']
    """

    return sync_detailed(
        schedule_id=schedule_id,
        client=client,
        datetime_start=datetime_start,
        datetime_end=datetime_end,
        overflow=overflow,
    ).parsed


async def asyncio_detailed(
    schedule_id: int,
    *,
    client: AuthenticatedClient,
    datetime_start: datetime.datetime,
    datetime_end: datetime.datetime,
    overflow: Union[Unset, bool] = True,
) -> Response[list["ScheduleShiftSchema"]]:
    """List Schedule Shifts

    Args:
        schedule_id (int):
        datetime_start (datetime.datetime):
        datetime_end (datetime.datetime):
        overflow (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['ScheduleShiftSchema']]
    """

    kwargs = _get_kwargs(
        schedule_id=schedule_id,
        datetime_start=datetime_start,
        datetime_end=datetime_end,
        overflow=overflow,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    schedule_id: int,
    *,
    client: AuthenticatedClient,
    datetime_start: datetime.datetime,
    datetime_end: datetime.datetime,
    overflow: Union[Unset, bool] = True,
) -> Optional[list["ScheduleShiftSchema"]]:
    """List Schedule Shifts

    Args:
        schedule_id (int):
        datetime_start (datetime.datetime):
        datetime_end (datetime.datetime):
        overflow (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['ScheduleShiftSchema']
    """

    return (
        await asyncio_detailed(
            schedule_id=schedule_id,
            client=client,
            datetime_start=datetime_start,
            datetime_end=datetime_end,
            overflow=overflow,
        )
    ).parsed
